=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-01-12 05:25:40 +0000
+++ bzrlib/branch.py	2006-01-15 20:59:32 +0000
@@ -15,35 +15,35 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+from cStringIO import StringIO
+import errno
+import os
 import shutil
 import sys
-import os
-import errno
 from warnings import warn
-from cStringIO import StringIO
 
 
 import bzrlib
+from bzrlib.config import TreeConfig
+from bzrlib.delta import compare_trees
+import bzrlib.errors as errors
+from bzrlib.errors import (BzrError, InvalidRevisionNumber, InvalidRevisionId,
+                           NoSuchRevision, HistoryMissing, NotBranchError,
+                           DivergedBranches, LockError, UnlistableStore,
+                           UnlistableBranch, NoSuchFile, NotVersionedError,
+                           NoWorkingTree)
 import bzrlib.inventory as inventory
-from bzrlib.trace import mutter, note
+from bzrlib.inventory import Inventory
 from bzrlib.osutils import (isdir, quotefn,
                             rename, splitpath, sha_file,
                             file_kind, abspath, normpath, pathjoin,
                             safe_unicode,
                             )
-import bzrlib.errors as errors
-from bzrlib.errors import (BzrError, InvalidRevisionNumber, InvalidRevisionId,
-                           NoSuchRevision, HistoryMissing, NotBranchError,
-                           DivergedBranches, LockError, UnlistableStore,
-                           UnlistableBranch, NoSuchFile, NotVersionedError,
-                           NoWorkingTree)
 from bzrlib.textui import show_status
+from bzrlib.trace import mutter, note
+from bzrlib.tree import EmptyTree, RevisionTree
 from bzrlib.revision import (Revision, is_ancestor, get_intervening_revisions,
                              NULL_REVISION)
-
-from bzrlib.delta import compare_trees
-from bzrlib.tree import EmptyTree, RevisionTree
-from bzrlib.inventory import Inventory
 from bzrlib.store import copy_all
 from bzrlib.store.text import TextStore
 from bzrlib.store.weave import WeaveStore
@@ -52,7 +52,6 @@
 from bzrlib.transport import Transport, get_transport
 import bzrlib.xml5
 import bzrlib.ui
-from config import TreeConfig
 
 
 BZR_BRANCH_FORMAT_4 = "Bazaar-NG branch, format 0.0.4\n"
@@ -508,6 +507,163 @@
         raise NotImplementedError('store_revision_signature is abstract')
 
 
+class BzrBranchFormat(object):
+    """An encapsulation of the initialization and open routines for a format.
+
+    Formats provide three things:
+     * An initialization routine,
+     * a format string,
+     * an open routine.
+
+    Formats are placed in an dict by their format string for reference 
+    during branch opening. Its not required that these be instances, they
+    can be classes themselves with class methods - it simply depends on 
+    whether state is needed for a given format or not.
+
+    Once a format is deprecated, just deprecate the initialize and open
+    methods on the format class. Do not deprecate the object, as the 
+    object will be created every time regardless.
+    """
+
+    _formats = {}
+    """The known formats."""
+
+    @classmethod
+    def find_format(klass, url):
+        """Return the format registered for URL."""
+        t = get_transport(url)
+        return klass._formats[t.get(".bzr/branch-format").read()]
+
+    def get_format_string(self):
+        """Return the ASCII format string that identifies this format."""
+        raise NotImplementedError(self.get_format_string)
+
+    def _find_modes(self, t):
+        """Determine the appropriate modes for files and directories.
+        
+        FIXME: When this merges into, or from storage,
+        this code becomes delgatable to a LockableFiles instance.
+
+        For now its cribbed and returns (dir_mode, file_mode)
+        """
+        try:
+            st = t.stat('.')
+        except errors.TransportNotPossible:
+            dir_mode = 0755
+            file_mode = 0644
+        else:
+            dir_mode = st.st_mode & 07777
+            # Remove the sticky and execute bits for files
+            file_mode = dir_mode & ~07111
+        if not BzrBranch._set_dir_mode:
+            dir_mode = None
+        if not BzrBranch._set_file_mode:
+            file_mode = None
+        return dir_mode, file_mode
+
+    def initialize(self, url):
+        """Create a branch of this format at url and return an open branch."""
+        t = get_transport(url)
+        from bzrlib.inventory import Inventory
+        from bzrlib.weavefile import write_weave_v5
+        from bzrlib.weave import Weave
+        
+        # Create an empty inventory
+        sio = StringIO()
+        # if we want per-tree root ids then this is the place to set
+        # them; they're not needed for now and so ommitted for
+        # simplicity.
+        bzrlib.xml5.serializer_v5.write_inventory(Inventory(), sio)
+        empty_inv = sio.getvalue()
+        sio = StringIO()
+        bzrlib.weavefile.write_weave_v5(Weave(), sio)
+        empty_weave = sio.getvalue()
+
+        # Since we don't have a .bzr directory, inherit the
+        # mode from the root directory
+        dir_mode, file_mode = self._find_modes(t)
+
+        t.mkdir('.bzr', mode=dir_mode)
+        control = t.clone('.bzr')
+        dirs = ['revision-store', 'weaves']
+        files = [('README', 
+            StringIO("This is a Bazaar-NG control directory.\n"
+            "Do not change any files in this directory.\n")),
+            ('branch-format', StringIO(self.get_format_string())),
+            ('revision-history', StringIO('')),
+            ('branch-name', StringIO('')),
+            ('branch-lock', StringIO('')),
+            ('pending-merges', StringIO('')),
+            ('inventory', StringIO(empty_inv)),
+            ('inventory.weave', StringIO(empty_weave)),
+            ('ancestry.weave', StringIO(empty_weave))
+        ]
+        control.mkdir_multi(dirs, mode=dir_mode)
+        control.put_multi(files, mode=file_mode)
+        mutter('created control directory in ' + t.base)
+        return BzrBranch(t)
+
+    def open(self, url, branch):
+        """Fill out the data in branch for the branch at url."""
+        raise NotImplementedError(self.open)
+
+    @classmethod
+    def register_format(klass, format):
+        klass._formats[format.get_format_string()] = format
+
+
+class BzrBranchFormat4(BzrBranchFormat):
+    """Bzr branch format 4.
+
+    This format has:
+     - flat stores
+     - TextStores for texts, inventories,revisions.
+
+    This format is deprecated: it indexes texts using a text it which is
+    removed in format 5; write support for this format has been removed.
+    """
+
+    def get_format_string(self):
+        """See BzrBranchFormat.get_format_string()."""
+        return BZR_BRANCH_FORMAT_4
+
+    def initialize(self, url):
+        """Format 4 branches cannot be created."""
+        raise NotImplementedError(self.initialize)
+
+
+class BzrBranchFormat5(BzrBranchFormat):
+    """Bzr branch format 5.
+
+    This format has:
+     - weaves for file texts and inventory
+     - flat stores
+     - TextStores for revisions and signatures.
+    """
+
+    def get_format_string(self):
+        """See BzrBranchFormat.get_format_string()."""
+        return BZR_BRANCH_FORMAT_5
+
+
+class BzrBranchFormat6(BzrBranchFormat):
+    """Bzr branch format 6.
+
+    This format has:
+     - weaves for file texts and inventory
+     - hash subdirectory based stores.
+     - TextStores for revisions and signatures.
+    """
+
+    def get_format_string(self):
+        """See BzrBranchFormat.get_format_string()."""
+        return BZR_BRANCH_FORMAT_6
+
+
+BzrBranchFormat.register_format(BzrBranchFormat4())
+BzrBranchFormat.register_format(BzrBranchFormat5())
+BzrBranchFormat.register_format(BzrBranchFormat6())
+
 class BzrBranch(Branch):
     """A branch stored in the actual filesystem.
 
@@ -563,7 +719,7 @@
         except UnlistableStore:
             raise UnlistableBranch(from_store)
 
-    def __init__(self, transport, init=False,
+    def __init__(self, transport, init=deprecated_nonce,
                  relax_version_check=False):
         """Create new branch object at a particular location.
 
@@ -584,13 +740,19 @@
         assert isinstance(transport, Transport), \
             "%r is not a Transport" % transport
         self._transport = transport
-        if init:
-            self._make_control()
+        if deprecated_passed(init):
+            warn("BzrBranch.__init__(..., init=XXX): The init parameter is "
+                 "deprecated as of bzr 0.8. Please use Branch.initialize().",
+                 DeprecationWarning)
+            if init:
+                # this is slower than before deprecation, oh well never mind.
+                # -> its deprecated.
+                self._initialize(transport.base)
         self._check_format(relax_version_check)
         self._find_modes()
 
         def get_store(name, compressed=True, prefixed=False):
-            relpath = self._rel_controlfilename(unicode(name))
+            relpath = self._rel_controlfilename(safe_unicode(name))
             store = TextStore(self._transport.clone(relpath),
                               dir_mode=self._dir_mode,
                               file_mode=self._file_mode,
@@ -627,8 +789,7 @@
     @staticmethod
     def _initialize(base):
         """Create a bzr branch in the latest format."""
-        t = get_transport(base)
-        return BzrBranch(t, init=True)
+        return BzrBranchFormat6().initialize(base)
 
     def __str__(self):
         return '%s(%r)' % (self.__class__.__name__, self._transport.base)
@@ -802,44 +963,6 @@
         if not self._set_file_mode:
             self._file_mode = None
 
-    def _make_control(self):
-        from bzrlib.inventory import Inventory
-        from bzrlib.weavefile import write_weave_v5
-        from bzrlib.weave import Weave
-        
-        # Create an empty inventory
-        sio = StringIO()
-        # if we want per-tree root ids then this is the place to set
-        # them; they're not needed for now and so ommitted for
-        # simplicity.
-        bzrlib.xml5.serializer_v5.write_inventory(Inventory(), sio)
-        empty_inv = sio.getvalue()
-        sio = StringIO()
-        bzrlib.weavefile.write_weave_v5(Weave(), sio)
-        empty_weave = sio.getvalue()
-
-        cfn = self._rel_controlfilename
-        # Since we don't have a .bzr directory, inherit the
-        # mode from the root directory
-        self._find_modes(u'.')
-
-        dirs = ['', 'revision-store', 'weaves']
-        files = [('README', 
-            "This is a Bazaar-NG control directory.\n"
-            "Do not change any files in this directory.\n"),
-            ('branch-format', BZR_BRANCH_FORMAT_6),
-            ('revision-history', ''),
-            ('branch-name', ''),
-            ('branch-lock', ''),
-            ('pending-merges', ''),
-            ('inventory', empty_inv),
-            ('inventory.weave', empty_weave),
-            ('ancestry.weave', empty_weave)
-        ]
-        self._transport.mkdir_multi([cfn(d) for d in dirs], mode=self._dir_mode)
-        self.put_controlfiles(files)
-        mutter('created control directory in ' + self._transport.base)
-
     def _check_format(self, relax_version_check):
         """Check this branch format is supported.
 

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2006-01-05 21:48:56 +0000
+++ bzrlib/symbol_versioning.py	2006-01-15 20:59:32 +0000
@@ -20,11 +20,17 @@
 The methods here allow for api symbol versioning.
 """
 
-__all__ = ['warn', 'set_warning_method', 'zero_seven']
+__all__ = ['deprecated_function',
+           'deprecated_method',
+           'deprecated_nonce',
+           'deprecated_passed',
+           'warn', 'set_warning_method', 'zero_seven',
+           ]
 
 from warnings import warn
 
 
+deprecated_nonce = "A deprecated parameter marker."
 zero_seven = "%s was deprecated in version 0.7."
 
 
@@ -84,6 +90,25 @@
     return method_decorator
 
 
+def deprecated_passed(parameter_value):
+    """Return True if parameter_value was used."""
+    # FIXME: it might be nice to have a parameter deprecation decorator. 
+    # it would need to handle positional and *args and **kwargs parameters,
+    # which means some mechanism to describe how the parameter was being
+    # passed before deprecation, and some way to deprecate parameters that
+    # were not at the end of the arg list. Thats needed for __init__ where
+    # we cannot just forward to a new method name.I.e. in the following
+    # examples we would want to have callers that pass any value to 'bad' be
+    # given a warning - because we have applied:
+    # @deprecated_parameter('bad', zero_seven)
+    #
+    # def __init__(self, bad=None)
+    # def __init__(self, bad, other)
+    # def __init__(self, **kwargs)
+    # RBC 20060116
+    return not parameter_value is deprecated_nonce
+
+
 def _decorate_docstring(callable, deprecation_version, label,
                         decorated_callable):
     docstring_lines = callable.__doc__.split('\n')

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2006-01-12 05:25:40 +0000
+++ bzrlib/tests/test_branch.py	2006-01-15 20:59:32 +0000
@@ -17,7 +17,8 @@
 import os
 import sys
 
-from bzrlib.branch import Branch, needs_read_lock, needs_write_lock, BzrBranch
+import bzrlib.branch as branch
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 from bzrlib.clone import copy_branch
 from bzrlib.commit import commit
 import bzrlib.errors as errors
@@ -404,7 +405,7 @@
     def test_get_set_default_initializer(self):
         old_initializer = Branch.get_default_initializer()
         # default is BzrBranch._initialize
-        self.assertEqual(BzrBranch._initialize, old_initializer)
+        self.assertEqual(branch.BzrBranch._initialize, old_initializer)
         def recorder(url):
             return "a branch %s" % url
         Branch.set_default_initializer(recorder)
@@ -414,3 +415,21 @@
         finally:
             Branch.set_default_initializer(old_initializer)
         self.assertEqual(old_initializer, Branch.get_default_initializer())
+
+
+class TestBzrBranchFormat(TestCaseInTempDir):
+    """Tests for the BzrBranchFormat facility."""
+
+    def test_find_format(self):
+        # is the right format object found for a branch?
+        # create a branch with a known format object
+        self.build_tree(["foo/", "bar/"])
+        def check_format(format, url):
+            format.initialize(url)
+            found_format = branch.BzrBranchFormat.find_format(url)
+            self.failUnless(isinstance(found_format, format.__class__))
+        check_format(branch.BzrBranchFormat5(), "foo")
+        check_format(branch.BzrBranchFormat6(), "bar")
+
+    def test_initialize_returns_branch(self):
+        self.failUnless(isinstance(branch.BzrBranchFormat5().initialize("."), branch.Branch))

=== modified file 'bzrlib/tests/test_symbol_versioning.py'
--- bzrlib/tests/test_symbol_versioning.py	2006-01-05 21:48:56 +0000
+++ bzrlib/tests/test_symbol_versioning.py	2006-01-15 20:59:32 +0000
@@ -89,3 +89,11 @@
             self.assertEqualDiff(expected_module, deprecated_callable.__module__)
         finally:
             symbol_versioning.set_warning_method(old_warning_method)
+    
+    def test_deprecated_passed(self):
+        self.assertEqual(True, symbol_versioning.deprecated_passed(None))
+        self.assertEqual(True, symbol_versioning.deprecated_passed(True))
+        self.assertEqual(True, symbol_versioning.deprecated_passed(False))
+        self.assertEqual(False,
+                         symbol_versioning.deprecated_passed(
+                            symbol_versioning.deprecated_nonce))

